function display_flux(range,opt,delay)
% DISPLAY_FLUX(RANGE,OPT,DELAY)
%    Read flux data files for indexes in the given RANGE and plot the resultant flux. If the range contains more than one index, the flux is summed and only one figure is displayed. When the DELAY between two data files is given, the intensity of the flux is computed according to the t0 time unit.
%    By default, the intensity of the flux is displayed and its direction is indicated by a vector field (OPT=8). This may be changed with the OPT parameter.
%
%    OPT: Optional parameter whose 5 least significant bits are treated as independent boolean flags (OPT=8 by default).
%      OPT=1 (bit-0): All figures are saved as PNG images.
%      OPT=2 (bit-1): Display as many figures as the number of indexes in the given RANGE.
%      OPT=4 (bit-2): The topography (corresponding to the last index in RANGE) is displayed instead of the intensity of the flux.
%      OPT=8 (bit-3): The vector field is displayed.
%      OPT=16 (bit-4): The streamlines are displayed.
%      OPT=32 (bit-5): The density of vectors and streamlines is doubled.
%
%    Examples: 
%       display_flux(1:10): Display intensity and direction of the flux for indexes from 1 to 10.
%       display_flux(10,0,1000): Display intensity of the flux in l0^3/t0 unit for index=10.
%       display_flux(1,4+8): Display flux as a vector field above the topography for index=1.
%       display_flux(1:5,1+2+8): Save and display five images of the flux for indexes from 1 to 5.
%       display_flux(1:20,1+4+8+32): Save and display the overall flux above the topography with a high density vector field for indexes from 1 to 20.

cpt=0;
str_data='';

if ~exist('opt','var')
  opt=8;
end

if ~exist('delay','var')
  delay=1;
  use_delay=0;
else
  use_delay=1;
end

[save_flag,sum_flag,alti_flag,int_flag,dir_flag,stream_flag]=get_flags(opt);
str_data='';
if save_flag
  if alti_flag
    str_data='_alti';
  end
  if use_delay
    str_data=sprintf('%s_time',str_data);
  end
end

for index=range
  %fprintf('index=%d\n',index);
  flux_file_ew=sprintf('FLUX_EW_%04d.data', index);
  flux_file_ns=sprintf('FLUX_NS_%04d.data', index);
  fx=load(flux_file_ew);
  fy=load(flux_file_ns);
  %% mean flux
  fxm(index)=mean(mean(fx));
  fym(index)=mean(mean(fy));
  fprintf('(%04d)   mean flux: %f (w->e), %f (n->s)\n', index, fxm(index)/delay, fym(index)/delay)

  if sum_flag
    if cpt==0
      fx_tot = fx;
      fy_tot = fy;
    else
      fx_tot = fx_tot+fx;
      fy_tot = fy_tot+fy;
    end
  else
    if alti_flag
      flux_file_al=sprintf('FLUX_AL_%04d.data', index);
      alti=load(flux_file_al);
      alti_fig(alti)
    end
    flux_fig(fx/delay,fy/delay,opt,use_delay,index)
    if save_flag
      img=sprintf('FLUX_%04d%s.png', index, str_data);
      fprintf('Figure saved as %s\n',img)
      print(img,'-dpng','-r250');
    end
  end
  
  cpt=cpt+1;
end

if sum_flag
  index1=min(range);
  index2=max(range);
  if use_delay
    delay_tot=(index2-index1+1)*delay;
  else
    delay_tot=1;
  end
  fprintf('(%04d-%04d)   mean flux: %f (w->s), %f (n->s)\n', index1, index2, mean(fxm(range))/delay, mean(fym(range))/delay)
  if alti_flag
    flux_file_al=sprintf('FLUX_AL_%04d.data', index);
    alti=load(flux_file_al);
    alti_fig(alti)
  end
  flux_fig(fx_tot/delay_tot,fy_tot/delay_tot,opt,use_delay)
  if save_flag
    if index1<index2
      img=sprintf('FLUX_%04d_%04d%s.png', index1, index2, str_data);
    else
      img=sprintf('FLUX_%04d%s.png', index1, str_data);
    end
    fprintf('Figure saved as %s\n',img)
    print(img,'-dpng','-r250');
  end
end

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function alti_fig(alti)
  figure
  imagesc(alti)
  %colormap(flipud(gray))
  colormap(copper)
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function flux_fig(fx,fy,opt,use_delay,idx)
  [save_flag,sum_flag,alti_flag,int_flag,dir_flag,stream_flag,density_flag]=get_flags(opt);
  
  if ~alti_flag
    figure
  else
    hold on
  end
  %axis off

  %% intensity
  if int_flag
    flux_intensity = sqrt(fx.*fx + fy.*fy);
    imagesc(flux_intensity)
    hold on
    %colormap(gray)
    %colormap(flipud(gray))
    %h=colorbar('East','Color','w');
    h=colorbar('EastOutside');
    if use_delay
      h.Title.String='Flux [l_0^3 t_0^{-1}]';
    else
      h.Title.String='Flux [l_0^3]';
    end
    %h.Title.Color='w';
    colormap(jet)
  end

  %xlim([1 size(fx,2)])
  %ylim([1 size(fx,1)])
  set(gca,'visible','off')
  set(gcf,'color','w')
  set(gca,'fontsize',6)
  set(gca,'xtick',[])
  set(gca,'ytick',[])
  %set(gca,'xcolor','w','ycolor','w','xtick',[],'ytick',[])
  daspect([1 1 1])

  if ~sum_flag
    title(sprintf('Index %04d',idx),'Visible','on');
  end
  
  %% vector field
  if dir_flag | stream_flag
    lx=size(fx,2);
    ly=size(fx,1);
    nb=40;
    if density_flag
      nb=nb*2;
    end
    step=floor(sqrt(lx*ly)/nb);
    %stepx=floor(lx/nb);
    %stepy=floor(ly/nb);
    vx=1:step:lx;
    vy=1:step:ly;
    [x,y] = meshgrid (vx,vy);
    qfx=fx(vy,vx);
    qfy=fy(vy,vx);
  end

  if dir_flag
    vfcol='w';
    %if int_flag
    %  vfcol='r';
    %end
    %if int_flag
      quiver(x,y,qfx,qfy,vfcol)%,'LineWidth',2)
    %else
    %  quiver(x,flipud(y),qfx,-qfy,'r')
    %end
    hold on
    xlim([0,lx])
    ylim([0,ly])
  end

  %% streamlines
  if stream_flag
    [sx,sy]=meshgrid(1:2*step:lx,1:2*step:ly);
    %if int_flag
      streamline(x,y,qfx,qfy,sx,sy)
    %else
    %  streamline(x,flipud(y),qfx,-qfy,sx,flipud(sy))
    %end
  end

  drawnow

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [save_flag,sum_flag,alti_flag,int_flag,dir_flag,stream_flag,density_flag]=get_flags(opt)
  save_flag=bitand(opt,1);
  sum_flag=~bitand(opt,2);
  if bitand(opt,4)
    alti_flag=1;
    int_flag=0;
  else
    alti_flag=0;
    int_flag=1;
  end
  dir_flag=bitand(opt,8);
  stream_flag=bitand(opt,16);
  density_flag=bitand(opt,32);
return

